## Post-estimation helper functions to supplement simcf:
## in-sample expected values (oprobitsimevSATE)
## and in-sample first differences, or SATEs (oprobitsimfdSATE)
##
## Christopher Adolph
## 6 March 2020    www.chrisadolph.com

##oprobitsimevSATE(xhyp, treatments, model, carbon, "SUPPORT", simbetas, cat=7,
##                 recode=list(c(1,2), c(3,4,5), c(6,7)), ci=0.95)


oprobitsimevSATE <- function(x, treatments, model, data, outcome,
                             simbetas, cat, recode=NULL, ci=0.95) {

    n <- nrow(data)
    if (is.null(recode)) 
        pe <- lower <- upper <- matrix(NA, nrow=nrow(x$x), ncol=cat)
    else
        pe <- lower <- upper <- matrix(NA, nrow=nrow(x$x), ncol=length(recode))
    for (i in 1:nrow(x$x)) {
        cdata <- data
        for (j in 1:length(treatments)) {
            cdata[[treatments[j]]] <- rep(x$x[[treatments[j]]][i], n) 
        }

        
        cf <- cfMake(model, cdata, nscen=nrow(data))        
        cf$x <- as.data.frame(cbind(cdata[[outcome]], model.matrix(model,cdata)[,-1]))
        names(cf$x)[1] <- outcome

        ev <- oprobitsimevSIM(cf, simbetas, cat=cat, constant=NA, recode=recode, ci=ci)
        sampleMean <- apply(ev$sim, 1:2, mean)
        pe[i,] <- apply(sampleMean, 2, mean)
        lower[i,] <- apply(sampleMean, 2, quantile, prob=(1-ci)/2)
        upper[i,] <- apply(sampleMean, 2, quantile, prob=(1+ci)/2)
    }
    res <- list(pe=pe, lower=lower, upper=upper)
    res
}



oprobitsimfdSATE <- function(x, treatments, model, data, outcome,
                             simbetas, cat, recode=NULL, ci=0.95) {

    n <- nrow(data)
    if (is.null(recode)) 
        pe <- lower <- upper <- matrix(NA, nrow=nrow(x$x), ncol=cat)
    else
        pe <- lower <- upper <- matrix(NA, nrow=nrow(x$x), ncol=length(recode))
    for (i in 1:nrow(x$x)) {
        cdata <- cdata0 <- data
        for (j in 1:length(treatments)) {
            cdata[[treatments[j]]] <- rep(x$x[[treatments[j]]][i], n)
            cdata0[[treatments[j]]] <- rep(x$xpre[[treatments[j]]][i], n) 
        }
        cf <- cfMake(model, cdata, nscen=nrow(data))
        cf$x <- as.data.frame(cbind(cdata[[outcome]], model.matrix(model,cdata)[,-1]))
        names(cf$x)[1] <- outcome
        cf$xpre <- as.data.frame(cbind(cdata0[[outcome]], model.matrix(model,cdata0)[,-1]))
        names(cf$xpre)[1] <- outcome
        
        ev <- oprobitsimevSIM(cf, simbetas, cat=cat, constant=NA, recode=recode, ci=ci)
        cf$x <- cf$xpre
        ev0 <- oprobitsimevSIM(cf, simbetas, cat=cat, constant=NA, recode=recode, ci=ci)
        sampleMean <- apply(ev$sim - ev0$sim, 1:2, mean)
        pe[i,] <- apply(sampleMean, 2, mean)
        lower[i,] <- apply(sampleMean, 2, quantile, prob=(1-ci)/2)
        upper[i,] <- apply(sampleMean, 2, quantile, prob=(1+ci)/2)
    }
    res <- list(pe=pe, lower=lower, upper=upper)
    res
}







oprobitsimevSIM <- function (x, b, ci = 0.95, constant = 1, cat = 3, recode = NULL) 
{
    if (cat < 3) {
        stop("cat must be at least 3 for ordered probit")
    }
    if (any(class(x) == "counterfactual") && !is.null(x$model)) {
        if (is.na(constant)) {
            reviseModel <- as.character(x$model)
            reviseModel <- as.formula(paste(reviseModel[2], reviseModel[1], 
                paste(reviseModel[3], "-1")))
            x$model <- reviseModel
        }
        x <- model.matrix(x$model, x$x)
    }
    else {
        if (any(class(x) == "list")) 
            x <- x$x
        if (is.data.frame(x)) 
            x <- as.matrix(x)
        if (!is.matrix(x)) {
            if (is.matrix(b)) {
                x <- t(x)
                if (!is.na(constant)) {
                  x <- append(x, 1, constant - 1)
                }
            }
            else {
                x <- as.matrix(x)
                if (!is.na(constant)) {
                  x <- appendmatrix(x, rep(1, nrow(x)), constant)
                }
            }
        }
        else {
            if (!is.na(constant)) {
                x <- appendmatrix(x, rep(1, nrow(x)), constant)
            }
        }
    }
    sims <- nrow(as.matrix(b))
    if (is.null(recode)) {
        res <- list(pe = NULL, lower = array(0, dim = c(nrow(x), 
            cat, length(ci))), upper = array(0, dim = c(nrow(x), 
            cat, length(ci))))
    }
    else {
        res <- list(pe = NULL, lower = array(0, dim = c(nrow(x), 
            length(recode), length(ci))), upper = array(0, dim = c(nrow(x), 
            length(recode), length(ci))))
    }

    if (!is.null(recode)) {
        simy0 <- array(NA, dim=c(nrow(b), length(recode), nrow(x)))
    } else {
        simy0 <- array(NA, dim=c(nrow(b), cat, nrow(x)))
    }

    for (i in 1:nrow(x)) {
        if (is.na(constant)) {
            simbeta <- b[, 1:(ncol(b) - cat + 1)]
            simtau <- b[, (ncol(b) - cat + 2):ncol(b), drop = FALSE]
        }
        else {
            simbeta <- b[, 1:(ncol(b) - cat + 2)]
            simtau <- b[, (ncol(b) - cat + 3):ncol(b), drop = FALSE]
        }
        if (is.vector(simbeta)) {
            simmu = as.matrix(simbeta) %*% x[i]
        }
        else {
            simmu <- simbeta %*% x[i, ]
        }
        simy <- matrix(NA, nrow = sims, ncol = cat)
        if (!is.na(constant)) {
            simy[, 1] <- pnorm(0, simmu)
            for (j in 2:(cat - 1)) {
                simy[, j] <- pnorm(simtau[, (j - 1)], simmu) - 
                  apply(simy[, 1:(j - 1), drop = FALSE], 1, sum)
            }
        }
        else {
            simy[, 1] <- pnorm(simtau[, 1], simmu)
            for (j in 2:(cat - 1)) {
                simy[, j] <- pnorm(simtau[, j], simmu) - apply(simy[, 
                  1:(j - 1), drop = FALSE], 1, sum)
            }
        }
        simy[, cat] <- 1 - apply(simy[, 1:(cat - 1), drop = FALSE], 
            1, sum)
        if (!is.null(recode)) 
            simy <- recategorize(simy, recode)
	simy0[,,i] <- simy    
        for (j in 1:ncol(simy)) {
            simy[, j] <- sort(simy[, j])
        }
        res$pe <- rbind(res$pe, apply(simy, 2, mean))
        length.simy <- nrow(simy)
        low <- up <- NULL
        for (k in 1:length(ci)) {
            res$lower[i, , k] <- cbind(low, simy[trunc((1 - ci[k])/2 * 
                length.simy), ])
            res$upper[i, , k] <- cbind(up, simy[trunc((1 - (1 - 
                ci[k])/2) * length.simy), ])
        }
    }
    if (length(ci) == 1) {
        if (nrow(x) == 1) {
            res$pe <- as.numeric(res$pe)
            res$lower <- as.numeric(res$lower)
            res$upper <- as.numeric(res$upper)
        }
        else {
            res$lower <- res$lower[, , 1, drop = TRUE]
            res$upper <- res$upper[, , 1, drop = TRUE]
        }
    }
    res$sim <- simy0
    res
}
